## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Original TechConquest Concept by Bhruic, Modified by Sevo
##

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import Popup as PyPopup
import pickle
import DataStorage
import CvCameraControls

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
data = DataStorage.DataStorage()
PyInfo = PyHelpers.PyInfo
dice = gc.getGame().getMapRand()


class NewForest:

	def onImprovementBuilt(self,argsList):
		iImprovement, iX, iY = argsList
		pPlot = CyMap().plot(iX,iY)
		if(iImprovement==gc.getInfoTypeForString('IMPROVEMENT_NEW_FOREST_4')):
			CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'Forest has matured!','AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),iX,iY,True,True)
			pPlot.setFeatureType(4, 1)
			pPlot.setImprovementType(-1)
			
	def updateAllPlots(self):
		newForest = gc.getInfoTypeForString("IMPROVEMENT_NEW_FOREST")
		newForest2 = gc.getInfoTypeForString("IMPROVEMENT_NEW_FOREST_2")
		newForest3 = gc.getInfoTypeForString("IMPROVEMENT_NEW_FOREST_3")
		newForest4 = gc.getInfoTypeForString("IMPROVEMENT_NEW_FOREST_4")
	
		pMap = CyMap()
		for x in range(pMap.getGridWidth()):
			for y in range(pMap.getGridHeight()):
				pPlot = pMap.plot(x,y)
				if (pPlot.getImprovementType() == newForest):
					if (dice.get(100, "Tech Chooser - TechConquest PYTHON") < 20 ):
						pPlot.setImprovementType(newForest2)
				elif (pPlot.getImprovementType() == newForest2):
					if (dice.get(100, "Tech Chooser - TechConquest PYTHON") < 20 ):
						pPlot.setImprovementType(newForest3)
				elif (pPlot.getImprovementType() == newForest3):
					if (dice.get(100, "Tech Chooser - TechConquest PYTHON") < 20 ):
						pPlot.setImprovementType(newForest4)
				elif (pPlot.getImprovementType() == newForest4):
					if (dice.get(100, "Tech Chooser - TechConquest PYTHON") < 25 ):
						pPlot.setImprovementType(-1)
						pPlot.setFeatureType(gc.getInfoTypeForString("FEATURE_FOREST"), 0)
						CyInterface().addMessage(pPlot.getOwner(),True,15,'Forest has matured!','AS2D_FEATUREGROWTH',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),x,y,True,True)
						
class CulturalDecay:

	def onEndGameTurn(self, argsList):
		pMap = CyMap()
		for iX in range(pMap.getGridWidth()):
			for iY in range(pMap.getGridHeight()):
				pPlot = pMap.plot(iX, iY)
				if (pPlot.countTotalCulture() > 0):
					for iLoop in range(gc.getMAX_PLAYERS()):
						pPlayer = gc.getPlayer(iLoop)
						if (pPlayer.isAlive()) or (pPlayer.isBarbarian()):
							iCulture = pPlot.getCulture(iLoop)
							iNumCultureCities = pPlot.getNumCultureRangeCities(iLoop)
							if (iCulture > 10) and (iNumCultureCities == 0):
								iCultureDecay = (int)(iCulture * 0.05)
								if (iCultureDecay < 1):
									iCultureDecay = 1
								pPlot.changeCulture(iLoop, -iCultureDecay, True)
					
					
class AncientTemple:

	def onImprovementBuilt(self,argsList):
		iImprovement, iX, iY = argsList
		pPlot = CyMap().plot(iX,iY)
		if(iImprovement==gc.getInfoTypeForString('IMPROVEMENT_TAPPED_TEMPLE')):
			
			CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'Ancient Temple has been explored!','',1,'Art/Atemple.dds',ColorTypes(8),iX,iY,True,True)
			pPlot.setImprovementType(gc.getInfoTypeForString("IMPROVEMENT_CITY_RUINS"))
			pPlot.setBonusType(-1)
			
			bonus = dice.get(40, "AncientTemple-Bonus1") + dice.get(40, "AncientTemple-Bonus2")
			pPlayer = gc.getPlayer(pPlot.getOwner())
			pPlayer.changeGold(bonus)
			CyInterface().addMessage(CyGame().getActivePlayer(),True,15,"%s gold discovered" % (bonus),'',1,'Art/Atemple.dds',ColorTypes(8),iX,iY,True,True)

class TechConquest:

	def onCityAcquired(self, argsList):
		owner,playerType,city,bConquest,bTrade = argsList
		if bConquest:
		
			loserTeam = gc.getTeam(owner)
			conqTeam = gc.getTeam(city.getOwner())
			listTechDiff = []
			conqMaxPoints = 0
			#tradeData = TradeData()
			#tradeData.ItemType = TradeableItems.TRADE_TECHNOLOGIES
			for iLoopTech in range(gc.getNumTechInfos()):
				#tradeData.iData = iLoopTech
				if (loserTeam.isHasTech(iLoopTech) == True) and (conqTeam.isHasTech(iLoopTech) == False):
					listTechDiff.append(iLoopTech)
				# This will keep track of the highest Asset Value technology HELD by the conquering civ
				if (conqTeam.isHasTech(iLoopTech)):
					Tpoints = gc.getTechInfo(iLoopTech).getAssetValue()
					if (Tpoints > conqMaxPoints):
						conqMaxPoints = Tpoints
			if len(listTechDiff) > 0:
			
				randTech = dice.get(len(listTechDiff), "Tech Chooser - TechConquest PYTHON")
				techData = gc.getTechInfo(listTechDiff[randTech])
				techCost = techData.getResearchCost()
				baseMultiplier = city.getPopulation()
				
				#SevoMess("TC:%s" %(techCost))
				
				# Base Points: 3.3% per population (city size 20 gives guaranteed half a technology)
				baseTechPointsAdded = (techCost / 30) * (baseMultiplier+1)
				
				# Divy up a random amount of extra tech points based on city size:
				# Size 20+ will give 150% random on top of 66%+ base
				if (baseMultiplier > 19):
					extraTechPointsAdded = dice.get(techCost, "Tech Chooser - TechConquest PYTHON") * 1.5
				# Size 10+ will give another 75% chance (on top of 33-66%)
				elif (baseMultiplier > 9):
					extraTechPointsAdded = dice.get((techCost), "Tech Chooser - TechConquest PYTHON") * 1
				# Size 1-10 will give 100% chance (on top of 6-33%)
				else:
					extraTechPointsAdded = int(dice.get((techCost), "Tech Chooser - TechConquest PYTHON") * 0.75)
					
				# Okay, now make some further modifications based on tech discrepancy ranges
				# conqMaxPoints is the highest Asset value tech held by our victors
				# We'll compare that to the Asset value of this technology and modify the tech points.
				# If the conq. team is more advanced than this tech, it'll improve the bonus, if the tech is more advanced
				# than the conq., then they don't gain as much (harder to assimilate)

				#  The first number is between +48 and -48, typically more like -24 to +24.
				# When multiplied by techCost/50, this will typically be between -50% and +50% of the total tech cost.
				AssetDifference = int(((conqMaxPoints - techData.getAssetValue()) * techCost)  / 50)
				#SevoMess("AD: %s, %s, %s, AND %s" %(conqMaxPoints, techData.getAssetValue(), techCost, AssetDifference))
				
				
				# Finally, modify this total for the game speed
				techRatio = gc.getGameSpeedInfo(gc.getGame().getGameSpeedType()).getResearchPercent()
				
				#Seems to need this modifier ( perhaps some kind of base modifier I'm missing somewhere?)
				techRatio = techRatio * 1.3
											
				# Now, clump it all together already
				totalTechPointsAdded = int((techRatio * (baseTechPointsAdded + extraTechPointsAdded + AssetDifference))/100)
				currentTechPoints = conqTeam.getResearchProgress(listTechDiff[randTech])
				#SevoMess("More: %s, %s, %s, AAAND %s" %(totalTechPointsAdded, currentTechPoints,techCost, techRatio))
				
				# Actually grant the technology here:
				totalTechPoints = int(currentTechPoints + totalTechPointsAdded)
				
				# Now cap it at the tech max.
				if (totalTechPoints > (techCost * techRatio)):
					# Adjust it up; the 1.1 is insurance that it will meet the tech cost.  Might be able to change this down the road.
					totalTechPoints = (techCost * techRatio * 1.1)
				
				#CyInterface().addMessage(city.getOwner(),True,20,"Trying %s" %(totalTechPoints),'',1,techData.getButton(),ColorTypes(10),city.getX(),city.getY(),True,True)	
				conqTeam.changeResearchProgress(listTechDiff[randTech], totalTechPoints, city.getOwner())
				
				if gc.getPlayer(city.getOwner()).isHuman():
					if (conqMaxPoints > techData.getAssetValue()):
						CyInterface().addMessage(city.getOwner(),True,20,"From the citizens of %s you aquire some of the primitive technology %s!" %(city.getName(),techData.getDescription()),'',1,techData.getButton(),ColorTypes(10),city.getX(),city.getY(),True,True)
					else:
						CyInterface().addMessage(city.getOwner(),True,20,"From the citizens of %s you learn some of the advanced secrets of %s!" %(city.getName(),techData.getDescription()),'',1,techData.getButton(),ColorTypes(10),city.getX(),city.getY(),True,True)	

								
class RealFort:
	def onUnitMove(self, argsList):
		'unit move'
		pPlot,pUnit = argsList

		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")
		
		if (pPlot.getImprovementType() == impFort):
			uct = pUnit.getUnitCombatType()
			if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
				pUnit.setHasPromotion(promFort1, true)
				pTeam = gc.getTeam(pUnit.getTeam())
				pFort = gc.getPromotionInfo(promFort2)
				if (pTeam.isHasTech(pFort.getTechPrereq())):
					pUnit.setHasPromotion(promFort2, true)
				pFort = gc.getPromotionInfo(promFort3)
				if (pTeam.isHasTech(pFort.getTechPrereq())):
					pUnit.setHasPromotion(promFort3, true)

		else:
			if (pUnit.isHasPromotion(promFort1)):
				pUnit.setHasPromotion(promFort1, false)
			if (pUnit.isHasPromotion(promFort2)):
				pUnit.setHasPromotion(promFort2, false)
			if (pUnit.isHasPromotion(promFort3)):
				pUnit.setHasPromotion(promFort3, false)
	
	
	def onImprovementBuilt(self, argsList):
		'Improvement Built'
		iImprovement, iX, iY = argsList

		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")

		pMap = CyMap()
		pPlot = pMap.plot(iX, iY)
		if (iImprovement == impFort):
			for iUnit in range(pPlot.getNumUnits()):
				pUnit = pPlot.getUnit(iUnit)
				uct = pUnit.getUnitCombatType()
				if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
					pUnit.setHasPromotion(promFort1, true)
					pTeam = gc.getTeam(pUnit.getTeam())
					pFort = gc.getPromotionInfo(promFort2)
					if (pTeam.isHasTech(pFort.getTechPrereq())):
						pUnit.setHasPromotion(promFort2, true)
					pFort = gc.getPromotionInfo(promFort3)
					if (pTeam.isHasTech(pFort.getTechPrereq())):
						pUnit.setHasPromotion(promFort3, true)
		else:
			for iUnit in range(pPlot.getNumUnits()):
				pUnit = pPlot.getUnit(iUnit)
				if (pUnit.isHasPromotion(promFort1)):
					pUnit.setHasPromotion(promFort1, false)
				if (pUnit.isHasPromotion(promFort2)):
					pUnit.setHasPromotion(promFort2, false)
				if (pUnit.isHasPromotion(promFort3)):
					pUnit.setHasPromotion(promFort3, false)

	def onTechAcquired(self, argsList):
		iTechType, iTeam, iPlayer, bAnnounce = argsList
	
		impFort = gc.getInfoTypeForString("IMPROVEMENT_FORT")
		promFort1 = gc.getInfoTypeForString("PROMOTION_FORT1")
		promFort2 = gc.getInfoTypeForString("PROMOTION_FORT2")
		promFort3 = gc.getInfoTypeForString("PROMOTION_FORT3")
		uctRecon = gc.getInfoTypeForString("UNITCOMBAT_RECON")
		uctArcher = gc.getInfoTypeForString("UNITCOMBAT_ARCHER")
		uctMounted = gc.getInfoTypeForString("UNITCOMBAT_MOUNTED")
		uctMelee = gc.getInfoTypeForString("UNITCOMBAT_MELEE")
		uctSiege = gc.getInfoTypeForString("UNITCOMBAT_SIEGE")
		uctGun = gc.getInfoTypeForString("UNITCOMBAT_GUN")
		uctArmor = gc.getInfoTypeForString("UNITCOMBAT_ARMOR")

		pFort2 = gc.getPromotionInfo(promFort2)
		pFort3 = gc.getPromotionInfo(promFort3)
		if (iTechType == pFort2.getTechPrereq()) or (iTechType == pFort3.getTechPrereq()):
			pMap = CyMap()
			for iX in range(pMap.getGridWidth()):
				for iY in range(pMap.getGridHeight()):
					pPlot = pMap.plot(iX, iY)
					if (pPlot.getImprovementType() == impFort):
						for iUnit in range(pPlot.getNumUnits()):
							pUnit = pPlot.getUnit(iUnit)
							if (pUnit.getTeam() == iTeam):
								uct = pUnit.getUnitCombatType()
								if (uct == uctRecon) or (uct == uctArcher) or (uct == uctMounted) or (uct == uctMelee) or (uct == uctSiege) or (uct == uctGun) or (uct == uctArmor):
									pUnit.setHasPromotion(promFort1, true)
									pTeam = gc.getTeam(pUnit.getTeam())
									if (iTechType == pFort2.getTechPrereq()):
										pUnit.setHasPromotion(promFort2, true)
									if (iTechType == pFort3.getTechPrereq()):
										pUnit.setHasPromotion(promFort3, true)
										
		
class SevoTraits:
	def __init__(self):
		self.NatlTrait = -1
		self.TolBuilding = -1
		self.TolTrait = -1

	def checkNewUnit(self, argsList, bForce):
		pUnit = argsList
				
		if (self.NatlTrait < 0):
			for iTrait in range(gc.getNumTraitInfos()):
				if (gc.getTraitInfo(iTrait).getTextKey() == 'TXT_KEY_TRAIT_NATIONALISTIC'):
					self.NatlTrait = iTrait
		
		if (not gc.getPlayer(pUnit.getOwner()).hasTrait(self.NatlTrait)):
			return
						
		promNatl = gc.getInfoTypeForString("PROMOTION_NATIONALISTIC")
		
		if (bForce == true):
			pUnit.setHasPromotion(promNatl, true)
		elif (pUnit.plot().getOwner() == pUnit.getOwner()):
			pUnit.setHasPromotion(promNatl, true)
			
			
	def onUnitMove(self, argsList):
		pPlot,pUnit = argsList

		if (self.NatlTrait < 0):
			for iTrait in range(gc.getNumTraitInfos()):
				if (gc.getTraitInfo(iTrait).getTextKey() == 'TXT_KEY_TRAIT_NATIONALISTIC'):
					self.NatlTrait = iTrait
		
		if (not gc.getPlayer(pUnit.getOwner()).hasTrait(self.NatlTrait)):
			return
			
		promNatl = gc.getInfoTypeForString("PROMOTION_NATIONALISTIC")
		
		if (pPlot.getOwner() == pUnit.getOwner()):
			pUnit.setHasPromotion(promNatl, true)
		else:
			if (pUnit.isHasPromotion(promNatl)):
				pUnit.setHasPromotion(promNatl, false)

	def checkCityForTrait(self, argsList):
		city = argsList
		
		if (self.TolTrait < 0):
			for iTrait in range(gc.getNumTraitInfos()):
				if (gc.getTraitInfo(iTrait).getTextKey() == 'TXT_KEY_TRAIT_TOLERANT'):
					self.TolTrait = iTrait
		
		if (gc.getPlayer(city.getOwner()).hasTrait(self.TolTrait)):
			if (city.getPopulation() > 2):
				if (self.TolBuilding < 0):
					for iBuild in range(gc.getNumBuildingInfos()):
						if (gc.getBuildingInfo(iBuild).getType() == 'BUILDING_TOLERANT'):
							self.TolBuilding = iBuild
				city.setHasRealBuilding(self.TolBuilding,1)
				#SevoMess("Added %s to %s" %(self.TolBuilding, city.getName()))
				
				
class JungleBurner:

	def onCityBuilt(self,argsList):
		city = argsList[0]
		X = city.getX()
		Y = city.getY()
		
		anyPlotChanged = 0
		
		for XX in range (X-1, X+2):
			for YY in range (Y-1, Y+2):
				pPlot = gc.getMap().plot(XX,YY)
				if (pPlot.getFeatureType() == gc.getInfoTypeForString("FEATURE_JUNGLE")):
					pPlot.setFeatureType(-1,-1)
					anyPlotChanged = anyPlotChanged + 1
		
		if (anyPlotChanged > 0):
				CyInterface().addMessage(CyGame().getActivePlayer(),True,15,'The jungle surrounding %s has been cleared.' %(city.getName()),'AS2D_FEATUREGROWTH',1,'Art/Interface/Buttons/TerrainFeatures/jungle.dds',ColorTypes(8),X,Y,True,True)
				
class RealSlavery:

	def onCombatResult(self, argsList):
		pWinningUnit,pLosingUnit = argsList
		pWinningPlayer = gc.getPlayer(pWinningUnit.getOwner())
		
		if (pWinningPlayer.getCivics(gc.getInfoTypeForString("CIVICOPTION_LABOR")) != gc.getInfoTypeForString("CIVIC_SLAVERY")):
			return
		
		pLosingPlayer = gc.getPlayer(pLosingUnit.getOwner())
		cLosingUnit = PyInfo.UnitInfo(pLosingUnit.getUnitType())
		
		if (cLosingUnit.isAnimal() == true):
			return
		elif not (pLosingUnit.isDead()):
			return
		elif (pLosingUnit.getNameKey() == 'TXT_KEY_UNIT_SLAVE'):
			return
		elif (pLosingUnit.getNameKey() == 'TXT_KEY_UNIT_SETTLER'):
			return
		elif (pLosingUnit.getNameKey() == 'TXT_KEY_UNIT_WORKER'):
			return
		elif (pLosingUnit.getNameKey() == 'TXT_KEY_UNIT_INDIAN_FAST_WORKER'):
			return
		elif (pLosingUnit.getNameKey() == 'TXT_KEY_UNIT_ABORIGINES_WOOMERA'):
			return	
		
		if (cLosingUnit.getDomainType() == gc.getInfoTypeForString("DOMAIN_LAND")):
			iRandom = dice.get(100, "RealSlavery-SlaveCaptureChance")
			if (iRandom < 20):	
		
				# Find the unit # of the slave unit
				self.slaveID = -1
				for i in range (gc.getNumUnitInfos()):
					if (gc.getUnitInfo(i).getTextKey() == 'TXT_KEY_UNIT_SLAVE'):
						self.slaveID = i
				if (self.slaveID == -1):
					sevoMess("Urp!  Couldn't find a slave ID!")
					return
				
				pNewUnit = pWinningPlayer.initUnit(self.slaveID, pWinningUnit.getX(), pWinningUnit.getY(), UnitAITypes.UNITAI_WORKER)
				CyInterface().addMessage(pWinningPlayer.getID(),True,15,"An enemy %s has been enslaved!" %(cLosingUnit.getDescription()),'AS2D_SLAVERCREATED',1,'Art/Interface/Buttons/units/slave.dds',ColorTypes(8),pWinningUnit.getX(),pWinningUnit.getY(),True,True)
				CyInterface().addMessage(pLosingPlayer.getID(),True,15,"Your %s has been enslaved!" %(cLosingUnit.getDescription()),'AS2D_SLAVERCREATED',1,'Art/Interface/Buttons/units/slave.dds',ColorTypes(7),pWinningUnit.getX(),pWinningUnit.getY(),True,True)		

	def onEndPlayerTurn(self, argsList):
		iGameTurn, iPlayer = argsList
		
		pPlayer = gc.getPlayer(iPlayer)
		
		self.playerSlaves[iPlayer] = 0
		self.playerCities = pPlayer.getNumCities()
		
		if (pPlayer.getCivics(gc.getInfoTypeForString("CIVICOPTION_LABOR")) == gc.getInfoTypeForString("CIVIC_SLAVERY")):
			return
		else: 
			# Not a slaver--check units to see if there are any slaves about.	
			unitsKilled = 0
			for iUnit in range (0,pPlayer.getNumUnits()+1):
				if (pPlayer.getUnit(iUnit).getNameKey() == 'TXT_KEY_UNIT_SLAVE'):
					pPlayer.getUnit(iUnit).kill(1,0)
					unitsKilled = 1
					
					# Here's the catch: We just killed this unit...thereby reducing the total # of units by 1, 
					# and shifting the entire player unit array.  Thus, we must re-run this index as it now
					# holds a new unit. So:
					iUnit = iUnit - 1
					
			if (unitsKilled == 1):
				CyInterface().addMessage(pPlayer.getID(),True,15,"Your slaves have been freed!",'AS2D_SLAVESFREED',1,'Art/Interface/Buttons/units/slave.dds',ColorTypes(8),-1,-1,True,True)
			
				
def SevoMess(message):
	CyInterface().addMessage(CyGame().getActivePlayer(),True,15,message,'AS2D_DISCOVERBONUS',1,'Art/Interface/Buttons/TerrainFeatures/Forest.dds',ColorTypes(8),1,1,True,True)	